package org.zenika.widget.client.datePicker;

import java.util.Date;

import org.zenika.widget.client.util.DateUtil;

import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;

/**
 * Main class of the DatePicker. It extends the TextBox widget and manages a Date object.
 * When it is clicked, it opens a PopupCalendar on which we can select a new date. <br>
 * Example of use :  <br>
 * <code>
 * DatePicker datePicker = new DatePicker();<br>
 * RootPanel.get().add(datePicker);<br>
 * </code>
 * You can specify a theme (see the CSS file DatePickerStyle.css) and
 * the date to initialize the date picker.
 * Enjoy xD
 * @author Nicolas Wetzel (nicolas.wetzel@zenika.com)
 * @author Jean-Philippe Dournel
 */
public class DatePicker extends TextBox implements ClickListener {

	private PopupCalendar popup;
	private Date selectedDate;
	// the oldest date that can be selected
	private Date oldestDate;
	// the youngest date that can be selected
	private Date youngestDate;
	private DateTimeFormat dateFormatter;
	
	{
		dateFormatter = DateUtil.getDateTimeFormat();
		popup = new PopupCalendar(this);
	}

	/**
	 * Default constructor. It creates a DatePicker which shows the current
	 * month.
	 */
	public DatePicker() {
		super();
		setText("");	
		sinkEvents(Event.ONCHANGE | Event.ONKEYPRESS);
		addClickListener(this);
	}

	/**
	 * Create a DatePicker which show a specific Date.
	 * @param selectedDate Date to show
	 */
	public DatePicker(Date selectedDate) {
		this();
		this.selectedDate = selectedDate;
		synchronizeFromDate();
	}
	
	/**
	 * Create a DatePicker which uses a specific theme.
	 * @param theme Theme name
	 */
	public DatePicker(String theme) {
		this();
		setTheme(theme);
	}
	
	/**
	 * Create a DatePicker which specifics date and theme.
	 * @param selectedDate Date to show
	 * @param theme Theme name
	 */
	public DatePicker(Date selectedDate, String theme) {
		this(selectedDate);
		setTheme(theme);
	}

	/**
	 * Return the Date contained in the DatePicker.
	 * @return The Date
	 */
	public Date getSelectedDate() {
		return selectedDate;
	}

	/**
	 * Set the Date of the datePicker and synchronize it with the display.
	 * @param value
	 */
	public void setSelectedDate(Date value) {
		this.selectedDate = value;
		synchronizeFromDate();
	}

	/**
	 * Return the theme name.
	 * @return Theme name
	 */
	public String getTheme() {
		return popup.getTheme();
	}

	/**
	 * Set the theme name.
	 * @param theme Theme name
	 */
	public void setTheme(String theme) {
		popup.setTheme(theme);
	}

	public void onBrowserEvent(Event event) {
		switch (DOM.eventGetType(event)) {
			case Event.ONBLUR:
				popup.hidePopupCalendar();
			break;
			case Event.ONCHANGE:
				parseDate();
			break;
			case Event.ONKEYPRESS:
				if (DOM.eventGetKeyCode(event) == 13) {
					parseDate();
					showPopup();
					break;
				}
		}
		super.onBrowserEvent(event);
	}
	
	/**
	 * @see com.google.gwt.user.client.ui.ClickListener#onClick(com.google.gwt.user.client.ui.Widget)
	 */
	public void onClick(Widget sender) {
		showPopup();
	}

	/**
	 * Display the date in the DatePicker.
	 */
	public void synchronizeFromDate() {
		if (this.selectedDate != null) {
			this.setText(dateFormatter.format(this.selectedDate));
		} else {
			this.setText("");
		}
	}

	/**
	 * Display the PopupCalendar.
	 */
	private void showPopup() {
		if (this.selectedDate != null) {
			popup.setDisplayedMonth(this.selectedDate);
		}
		popup.setPopupPosition(this.getAbsoluteLeft()+150, this.getAbsoluteTop());
		popup.displayMonth();
		doAfterShowPopup(popup);
	}
	
	/**
	 * Call when the calendar pop-up is shown. Can be overwritten.
	 * @param popup
	 */
	protected void doAfterShowPopup(PopupCalendar popup) {
		// nothing special to do
		
		// popup position can be updated so that it is displayed "above" the TexrBox
		// popup.setPopupPosition(popup.getPopupLeft(), popup.getPopupTop() - 150);
		
		// there is also a known bug if the DatePicker is in a modal MyGWT Dialog
		// DOM.setIntStyleAttribute(popup.getElement(), "zIndex", MyDOM.getZIndex());
		// Element p = DOM.getParent(popup.getElement());
		// int index = DOM.getChildIndex(p, popup.getElement());
		// p = DOM.getChild(p, --index);
		// DOM.setIntStyleAttribute(p, "zIndex", MyDOM.getZIndex() - 2);
	}

	/**
	 * Parse the date entered in the DatePicker.
	 */
	private void parseDate() {
		try {
			selectedDate = dateFormatter.parse(getText());
		} catch (IllegalArgumentException e) {
			// Do something ?
		}
		synchronizeFromDate();
	}

	public Date getOldestDate() {
		return oldestDate;
	}

	public void setOldestDate(Date oldestDate) {
		this.oldestDate = oldestDate;
	}

	public Date getYoungestDate() {
		return youngestDate;
	}

	public void setYoungestDate(Date youngestDate) {
		this.youngestDate = youngestDate;
	}

}
